@@ -30,46 +30,62 @@ Reference:
 '''
 
 _SEQDB = {
-    '< <NUL>'            : 'NUL / Ctrl-@,Ctrl-SP,Ctrl-2',
-    '> <NUL>'            : 'NUL / null character',
+    """ input """
+    '< <NUL>'            : 'NUL / Ctrl-@, Ctrl-SP, Ctrl-2',
+    '< <SOH>'            : 'SOH / Ctrl-A',
+    '< <STX>'            : 'STX / Ctrl-B',
+    '< <ETX>'            : 'ETX / Ctrl-C',
+    '< <EOT>'            : 'EOT / Ctrl-D',
+    '< <ENQ>'            : 'ENQ / Ctrl-E',
+    '< <ACK>'            : 'ACK / Ctrl-F',
     '< <BEL>'            : 'BEL / Ctrl-G',
-    '> <BEL>'            : 'BEL / bell',
-    '> ESC P<ST>'        : 'DCS / device control string',
-    '> ESC ]<ST>'        : 'OSC / operating system command',
-    '> ESC ]0<ST>'       : 'OSC 0 / set icon name and window title',
-    '> ESC ]1<ST>'       : 'OSC 1 / set icon name',
-    '> ESC ]2<ST>'       : 'OSC 2 / set window title',
-    '> ESC ]4<ST>'       : 'OSC 4 / get or set color palette',
-    '> ESC ]9<ST>'       : 'OSC 9 / Growl integration (iTerm2)',
-    '> ESC ]10<ST>'      : 'OSC 10 / get or set foreground color',
-    '> ESC ]11<ST>'      : 'OSC 11 / get or set background color',
-    '> ESC ]12<ST>'      : 'OSC 12 / get or set cursor color',
-    '> ESC ]13<ST>'      : 'OSC 13 / get or set mouse foreground color',
-    '> ESC ]14<ST>'      : 'OSC 13 / get or set mouse background color',
-    '> ESC ]51<ST>'      : 'OSC 51 / reserved for Emacs',
-    '> ESC ]52<ST>'      : 'OSC 52 - PASTE64 / base64 clipboard read/write operation (xterm)',
-    '> ESC ^<ST>'        : 'PM / private message',
-    '> ESC _<ST>'        : 'APC / application program command',
-    '> ESC V'            : 'SPA / start of guarded area',
-    '> ESC W'            : 'EPA / end of guarded area',
-    '> ESC X<ST>'        : 'SOS / start of string',
+    '< <BS>'             : 'BS / Ctrl-H,may be Backspace key',
+    '< <HT>'             : 'HT / Ctrl-I,may be Tab key',
+    '< <NL>'             : 'NL / Ctrl-J',
+    '< <VT>'             : 'VT / Ctrl-K',
+    '< <NP>'             : 'NP / Ctrl-L',
+    '< <CR>'             : 'CR / Ctrl-M',
+    '< <SO>'             : 'SO / Ctrl-N',
+    '< <DLE>'            : 'DLE / Ctrl-P',
+    '< <DC1>'            : 'DC1 / Ctrl-Q',
+    '< <DC2>'            : 'DC2 / Ctrl-R',
+    '< <DC3>'            : 'DC3 / Ctrl-S',
+    '< <DC4>'            : 'DC4 / Ctrl-T',
+    '< <NAK>'            : 'NAK / Ctrl-U',
+    '< <SYN>'            : 'SYN / Ctrl-V',
+    '< <ETB>'            : 'ETB / Ctrl-W',
+    '< <CAN>'            : 'CAN / Ctrl-X',
+    '< <EM>'             : 'EM / Ctrl-Y',
+    '< <SUB>'            : 'SUB / Ctrl-Z',
+    '< <ESC>'            : 'ESC / Ctrl-[, Ctrl-3,  or Escape key',
+    '< <FS> '            : 'SUB / Ctrl-\\, Ctrl-4',
+    '< <GS>'             : 'GS / Ctrl-], Ctrl-5',
+    '< <RS>'             : 'RS / Ctrl-^, Ctrl-6',
+    '< <US>'             : 'US / Ctrl-_, Ctrl-7',
+    '< <DEL>'            : 'DEL / Ctrl-8 or Delete key',
+    '< CSI A'            : 'Cursor key(normal keypad): up arrow',
+    '< CSI B'            : 'Cursor key(normal keypad): down arrow',
+    '< CSI C'            : 'Cursor key(normal keypad): right arrow',
+    '< CSI D'            : 'Cursor key(normal keypad): left arrow',
     '< CSI [0]M'         : 'xterm normal mouse reporting, following 3 bytes mean (code, row, col)',
-    '< CSI M'            : 'URXVT (1015) mouse reporting (code,row,col)',
-    '< CSI [3]M'         : 'URXVT (1015) mouse reporting (code=(%s-32),row=%s,col=%s)',
+    '< CSI M'            : 'urxvt (1015) mouse reporting (code,row,col)',
+    '< CSI [3]M'         : 'urxvt (1015) mouse reporting (code=(%s-32),row=%s,col=%s)',
     '< CSI <[3]M'        : 'SGR (1006) mouse reporting (code=%s,row=%s,col=%s)',
     '< CSI <M'           : 'SGR (1006) mouse reporting (code,row,col)',
     '< CSI <[3]m'        : 'SGR (1006) mouse reporting, button up (code=%s,row=%s,col=%s)',
     '< CSI <m'           : 'SGR (1006) mouse reporting, button up (code, row, col)',
+    '< CSI [0]I'         : 'Focus mode reporting: focus in',
+    '< CSI [0]O'         : 'Focus mode reporting: focus out',
     '< CSI [3]R'         : 'DSR-DECXCPR(cursor position report) response, DEC specific: (row=%s,col=%s,page=%s)',
     '< CSI [2]R'         : 'DSR-CPR(cursor position report) response: (row=%s,col=%s)',
     '< CSI R'            : 'DSR-CPR(cursor position report) response',
     '< CSI 0n'           : 'DSR-OS(operating status report) response: "good status"',
     '< CSI 3n'           : 'DSR-OS(operating status report) response: "it has a malfunction"',
     '< CSI ?c'           : 'DA1 Response',
+    '< CSI ?1;0c'        : 'DA1 Response: VT100 with no options (could be WizConsole VT100 mode)',
     '< CSI ?1;2c'        : 'DA1 Response: VT100 with AVO (could be a VT102/rxvt/rxvt-unicode/konsole/kterm/mlterm/iTerm2/tmux/aterm/wterm/Terminal.app(Mac)/mrxvt/MinTTY/TeraTerm/ck)',
-    '< CSI ?1;0c'        : 'DA1 Response: VT100 with no options',
     '< CSI ?6c'          : 'DA1 Response: VT102 (could be linux xvt/console/PuTTY/Terminal.app(GNUStep)/yaft)',
-    '< CSI ?62c'         : 'DA1 Response: VT200 family (could be mosh)',
+    '< CSI ?62c'         : 'DA1 Response: VT200 family (could be mosh/WizConsole VT200 mode)',
     '< CSI ?62;9;c'      : 'DA1 Response: VT200 family (could be gnome-terminal)',
     '< CSI ?64;1;2;6;9;15;21;22c':
     'DA1 Response: VT400 family (could be xterm pl>=280)',
@@ -77,8 +93,10 @@ _SEQDB = {
     'DA2 Response: VT400 family (could be tanasinn)',
     '< CSI ?65;1;2;3,4;6;8;9;18;21;22;29;42;44c':
     'DA1 Response: VT500 family (could be RLogin)',
+    '< CSI ?65;1;2;3;4;6;8;9;18;21;22;29;42;44c':
+    'DA1 Response: VT500 family (could be RLogin)',
     '< CSI >c'           : 'DA2 Response',
-    '< CSI >0;[*]c'      : 'DA2 Response: VT100',
+    '< CSI >0;[2]c'      : 'DA2 Response: VT100, Pv=%d, keyboard type=%d',
     '< CSI >0;95;c'      : 'DA2 Response: xterm pattch#95 (could be iTerm/iTerm2)',
     '< CSI >0;95;0c'     : 'DA2 Response: xterm pattch#95 (could be tmux/)',
     '< CSI >0;95;1c'     : 'DA2 Response: xterm pattch#95 (could be avaliable mouse)',
@@ -94,19 +112,19 @@ _SEQDB = {
     '< CSI >0;277;0c'    : 'DA2 Response: VT100 (could be xterm patch#277 or tanasinn)',
     '< CSI >0;278;0c'    : 'DA2 Response: VT100 (could be xterm patch#278)',
     '< CSI >0;279;0c'    : 'DA2 Response: VT100 (could be xterm patch#279)',
-    '< CSI >1;[*]c'      : 'DA2 Response: VT220',
-    '< CSI >1;10;0c'     : 'DA2 Response: VT220 (could be mosh)',
+    '< CSI >1;[2]c'      : 'DA2 Response: VT220, Pv=%d, keyboard type=%d',
+    '< CSI >1;10;0c'     : 'DA2 Response: VT220 (could be mosh/WizConsole VT200 mode)',
     '< CSI >1;96;0c'     : 'DA2 Response: VT220 (could be xterm pattch#96, mlterm)',
     '< CSI >1;2403;0c'   : 'DA2 Response: VT220 (could be VTE, such as gnome-terminal)',
     '< CSI >1;2802;0c'   : 'DA2 Response: VT220 (could be VTE, such as gnome-terminal)',
-    '< CSI >2;[2]c'      : 'DA2 Response: VT240',
-    '< CSI >18;[2]c'     : 'DA2 Response: VT330',
-    '< CSI >19;[2]c'     : 'DA2 Response: VT340',
-    '< CSI >19;[2]c'     : 'DA2 Response: VT340',
-    '< CSI >24;[2]c'     : 'DA2 Response: VT320',
-    '< CSI >32;[2]c'     : 'DA2 Response: VT382',
+    '< CSI >2;[2]c'      : 'DA2 Response: VT240, Pv=%d, keyboard type=%d',
+    '< CSI >18;[2]c'     : 'DA2 Response: VT330, Pv=%d, keyboard type=%d',
+    '< CSI >19;[2]c'     : 'DA2 Response: VT340, Pv=%d, keyboard type=%d',
+    '< CSI >19;[2]c'     : 'DA2 Response: VT340, Pv=%d, keyboard type=%d',
+    '< CSI >24;[2]c'     : 'DA2 Response: VT320, Pv=%d, keyboard type=%d',
+    '< CSI >32;[2]c'     : 'DA2 Response: VT382, Pv=%d, keyboard type=%d',
     '< CSI >32;100;2c'   : 'DA2 Response: VT382 (could be TeraTerm)',
-    '< CSI >41;[2]c'     : 'DA2 Response: VT420',
+    '< CSI >41;[2]c'     : 'DA2 Response: VT420, Pv=%d, keyboard type=%d',
     '< CSI >41;280;0c'   : 'DA2 Response: VT420 (could be xterm patch#280)',
     '< CSI >41;281;0c'   : 'DA2 Response: VT420 (could be xterm patch#281)',
     '< CSI >41;282;0c'   : 'DA2 Response: VT420 (could be xterm patch#282)',
@@ -120,16 +138,20 @@ _SEQDB = {
     '< CSI >41;290;0c'   : 'DA2 Response: VT420 (could be xterm patch#290)',
     '< CSI >41;291;0c'   : 'DA2 Response: VT420 (could be xterm patch#291)',
     '< CSI >41;292;0c'   : 'DA2 Response: VT420 (could be xterm patch#292)',
-    '< CSI >61;[2]c'     : 'DA2 Response: VT510',
-    '< CSI >64;[2]c'     : 'DA2 Response: VT520',
-    '< CSI >65;[2]c'     : 'DA2 Response: VT525',
+    '< CSI >61;[2]c'     : 'DA2 Response: VT510, Pv=%d, keyboard type=%d',
+    '< CSI >64;[2]c'     : 'DA2 Response: VT520, Pv=%d, keyboard type=%d',
+    '< CSI >65;[2]c'     : 'DA2 Response: VT525, Pv=%d, keyboard type=%d',
     '< CSI >65;100;1c'   : 'DA1 Response: VT500 family (could be RLogin)',
-    '< CSI >77;[2]c'     : 'DA2 Response: could be MinTTY',
+    '< CSI >77;[2]c'     : 'DA2 Response: could be MinTTY, Pv=%d, keyboard type=%d',
+    '< CSI >77;[1];c'    : 'DA2 Response: could be MinTTY, Pv=%d',
     '< CSI >77;10003;c'  : 'DA2 Response: could be MinTTY 1.0.3',
-    '< CSI >82;[*]c'     : 'DA2 Response: could be mrxvt',
+    '< CSI >77;10101;c'  : 'DA2 Response: could be MinTTY 1.1.1',
+    '< CSI >77;10102;c'  : 'DA2 Response: could be MinTTY 1.1.2',
+    '< CSI >77;10103;c'  : 'DA2 Response: could be MinTTY 1.1.3',
+    '< CSI >82;[2]c'     : 'DA2 Response: could be mrxvt, Pv=%d, keyboard type=%d',
     '< CSI >82;20710;0c' : 'DA2 Response: could be rxvt/ck',
     '< CSI >82;0.5.4;0c' : 'DA2 Response: could be mrxvt 0.5.4',
-    '< CSI >83;[*]c'     : 'DA2 Response: could be GNU Screen',
+    '< CSI >83;[2]c'     : 'DA2 Response: could be GNU Screen, Pv=%d, keyboard type=%d',
     '< CSI >83;40000;0c' : 'DA2 Response: could be GNU Screen 4.0.0',
     '< CSI >83;40001;0c' : 'DA2 Response: could be GNU Screen 4.0.1',
     '< CSI >83;40002;0c' : 'DA2 Response: could be GNU Screen 4.0.2',
@@ -146,6 +168,30 @@ _SEQDB = {
     '< CSI 24~'          : 'F12 key (xterm)',
     '< CSI 200~'         : 'Start sequense of bracketed paste',
     '< CSI 201~'         : 'Terminate bracketed paste',
+
+    """ output """
+    '> <NUL>'            : 'NUL / null character',
+    '> <BEL>'            : 'BEL / bell',
+    '> ESC P<ST>'        : 'DCS / device control string',
+    '> ESC ]<ST>'        : 'OSC / operating system command',
+    '> ESC ]0<ST>'       : 'OSC 0 / set icon name and window title',
+    '> ESC ]1<ST>'       : 'OSC 1 / set icon name',
+    '> ESC ]2<ST>'       : 'OSC 2 / set window title',
+    '> ESC ]4<ST>'       : 'OSC 4 / get or set color palette',
+    '> ESC ]9<ST>'       : 'OSC 9 / Growl integration (iTerm2)',
+    '> ESC ]10<ST>'      : 'OSC 10 / get or set foreground color',
+    '> ESC ]11<ST>'      : 'OSC 11 / get or set background color',
+    '> ESC ]12<ST>'      : 'OSC 12 / get or set cursor color',
+    '> ESC ]13<ST>'      : 'OSC 13 / get or set mouse foreground color',
+    '> ESC ]14<ST>'      : 'OSC 13 / get or set mouse background color',
+    '> ESC ]51<ST>'      : 'OSC 51 / reserved for Emacs',
+    '> ESC ]52<ST>'      : 'OSC 52 - PASTE64 / base64 clipboard read/write operation (xterm)',
+    '> ESC ]8900<ST>'    : 'OSC 8890 - GWREPT / glyph width report (yaft)',
+    '> ESC ^<ST>'        : 'PM / private message',
+    '> ESC _<ST>'        : 'APC / application program command',
+    '> ESC V'            : 'SPA / start of guarded area',
+    '> ESC W'            : 'EPA / end of guarded area',
+    '> ESC X<ST>'        : 'SOS / start of string',
     '> ESC D'            : 'IND / moves the cursor down one line in the same column',
     '> ESC E'            : 'NEL / moves the cursor to the first position on the next line',
     '> ESC H'            : 'HTS / sets a horizontal tab stop at the column where the cursor is. ',
@@ -319,6 +365,19 @@ _SEQDB = {
     '> CSI ?12h'         : 'DECSET 12 / blinking cursor (att610)',
     '> CSI ?12;25h'      : 'DECSET 12;25 / visible and blinking cursor',
     '> CSI ?25h'         : 'DECSET 25 - DECTCEM / show cursor',
+    '> CSI ?30h'         : 'DECSET 30 / show scroll bar (rxvt)',
+    '> CSI ?35h'         : 'DECSET 35 / enable font-shifting functions (rxvt)',
+    '> CSI ?38h'         : 'DECSET 38 - DECTEK / enter Tektronix mode',
+    '> CSI ?40h'         : 'DECSET 40 / disallow 80 -> 132 mode',
+    '> CSI ?41h'         : 'DECSET 41 / more(1) fix',
+    '> CSI ?42h'         : 'DECSET 42 / enable nation replacement character set',
+    '> CSI ?44h'         : 'DECSET 44 / turn on margin bell',
+    '> CSI ?45h'         : 'DECSET 45 / reverse wraparound mode',
+    '> CSI ?46h'         : 'DECSET 46 / start logging',
+    '> CSI ?47h'         : 'DECSET 47 / use alternate screen buffer',
+    '> CSI ?66h'         : 'DECSET 66 - DECNKM / application keypad',
+    '> CSI ?69h'         : 'DECSET 69 - DECLRM / enable left and right margin mode',
+    '> CSI ?95h'         : 'DECSET 95 - DECNCSM / do not clear screen when DECCOLM is set/reset',
     '> CSI ?1000h'       : 'DECSET 1000 / enable xterm normal mouse tracking',
     '> CSI ?1001h'       : 'DECSET 1001 / enable highlight mouse tracking',
     '> CSI ?1002h'       : 'DECSET 1002 / enable button mouse tracking',
@@ -327,10 +386,16 @@ _SEQDB = {
     '> CSI ?1005h'       : 'DECSET 1005 / enable UTF8 mouse mode',
     '> CSI ?1006h'       : 'DECSET 1006 / enable SGR mouse mode',
     '> CSI ?1007h'       : 'DECSET 1007 / enable alternate scroll mode',
-    '> CSI ?1047h'       : 'DECSET 1047 / use alternate screen buffer',
+    '> CSI ?1010h'       : 'DECSET 1010 / scroll to bottom on tty output (rxvt)',
+    '> CSI ?1011h'       : 'DECSET 1011 / scroll to bottom on key press (rxvt)',
+    '> CSI ?1015h'       : 'DECSET 1015 / enable urxvt mouse mode',
+    '> CSI ?1047h'       : 'DECSET 1047 / use alternate screen buffer, clearing screen first if in the main screen',
     '> CSI ?1049h'       : 'DECSET 1049 / save cursor as in DECSC and use alternate screen buffer',
     '> CSI ?2004h'       : 'DECSET 2004 / enable bracketed paste mode',
-    '> CSI ?7700h'       : 'DECSET 7700 / enable ambiguous reporting (mintty)',
+    '> CSI ?7700h'       : 'DECSET 7700 / enable ambiguous reporting (MinTTY)',
+    '> CSI ?7727h'       : 'DECSET 7727 / application escape keycode mode (MinTTY)',
+    '> CSI ?7728h'       : 'DECSET 7728 / treat ^\ as normal escape keycode (MinTTY)',
+    '> CSI ?7766h'       : 'DECSET 7766 / show scroll bar (MinTTY)',
     '> CSI i'            : 'MC',
     '> CSI ?i'           : 'MC - DEC Specific',
     '> CSI l'            : 'RM',
@@ -347,6 +412,19 @@ _SEQDB = {
     '> CSI ?10l'         : 'DECRST 10 / hide toolbar (rxvt)',
     '> CSI ?12l'         : 'DECRST 12 / steady cursor',
     '> CSI ?25l'         : 'DECRST 25 - DECTCEM / hide cursor',
+    '> CSI ?30l'         : 'DECRST 30 / hide scroll bar (rxvt)',
+    '> CSI ?35l'         : 'DECRST 35 / disable font-shifting functions (rxvt)',
+    '> CSI ?38l'         : 'DECRST 38 - DECTEK / exit Tektronix mode',
+    '> CSI ?40l'         : 'DECRST 40 / allow 80 -> 132 mode',
+    '> CSI ?41l'         : 'DECRST 41 / no more(1) fix',
+    '> CSI ?42l'         : 'DECRST 42 / disable nation replacement character set',
+    '> CSI ?44l'         : 'DECRST 44 / turn off margin bell',
+    '> CSI ?45l'         : 'DECRST 45 / no reverse wraparound mode',
+    '> CSI ?46l'         : 'DECRST 46 / stop logging',
+    '> CSI ?47l'         : 'DECRST 47 / use normal screen buffer',
+    '> CSI ?66l'         : 'DECRST 66 - DECNKM / numeric keypad',
+    '> CSI ?69l'         : 'DECRST 69 - DECLRM / disable left and right margin mode',
+    '> CSI ?95l'         : 'DECRST 95 - DECNCSM / clear screen when DECCOLM is set/reset',
     '> CSI ?1000l'       : 'DECRST 1000 / disable xterm normal mouse mode',
     '> CSI ?1001l'       : 'DECRST 1001 / disable highlight mouse tracking',
     '> CSI ?1002l'       : 'DECRST 1002 / disable button mouse tracking',
@@ -355,10 +433,16 @@ _SEQDB = {
     '> CSI ?1005l'       : 'DECRST 1005 / disable UTF8 mouse mode',
     '> CSI ?1006l'       : 'DECRST 1006 / disable SGR mouse mode',
     '> CSI ?1007l'       : 'DECRST 1007 / disable alternate scroll mode',
-    '> CSI ?1047l'       : 'DECRST 1047 / use normal screen buffer',
+    '> CSI ?1010l'       : 'DECRST 1010 / don\'t scroll to bottom on tty output (rxvt)',
+    '> CSI ?1011l'       : 'DECRST 1011 / don\'t scroll to bottom on key press (rxvt)',
+    '> CSI ?1015l'       : 'DECRST 1015 / disable urxvt mouse mode',
+    '> CSI ?1047l'       : 'DECRST 1047 / use normal screen buffer, clearing screen first if in the alt screen',
     '> CSI ?1049l'       : 'DECRST 1049 / use normal screen buffer and restore cursor as in DECRC',
     '> CSI ?2004l'       : 'DECRST 2004 / reset bracketed paste mode',
     '> CSI ?7700l'       : 'DECRST 7700 / disable ambiguous reporting (mintty)',
+    '> CSI ?7727l'       : 'DECRST 7727 / normal escape keycode mode (MinTTY)',
+    '> CSI ?7728l'       : 'DECRST 7728 / treat ^[ as normal escape keycode (MinTTY)',
+    '> CSI ?7766l'       : 'DECRST 7766 / hide scroll bar (MinTTY)',
     '> CSI m'            : 'SGR / select graphics rendition',
     '> CSI 0m'           : 'SGR 0 / reset',
     '> CSI 00m'          : 'SGR 0 / reset',
@@ -459,3 +543,26 @@ _SEQDB = {
 def get():
     return _SEQDB
 
+def _test():
+    """
+    >>> _test()
+    'ok'
+    """
+    import re
+    pattern = re.compile("\\[([0-9]+)\\]")
+    for key, value in get().items():
+        if "[*]" in key:
+            raise Exception("[*] syntax is deprecated.")
+        match = re.search(pattern, key) 
+        if not match is None:
+            n = match.group(1)
+            try:
+                value % tuple([0] * int(n))
+            except TypeError, e:
+                raise Exception("invalid entry: (%s, %s)" % (key, value))
+    return "ok"
+
+if __name__ == "__main__":
+   import doctest
+   doctest.testmod()
+
